@@ -10,7 +10,7 @@ fn main() -> io::Result<()> {
     slings::block_on(async {
         let addr: SocketAddr = "127.0.0.1:8080".parse().unwrap();
         let listener = TcpListener::bind(addr).await?;
-        println!("server start listen on 127.0.0.1:8080");
+        println!("server start listen on {:?}", listener.local_addr());
         loop {
             let (mut stream, addr) = listener.accept().await?;
             println!("accept stream from addr: {:?}", addr);
