@@ -13,6 +13,7 @@ use slab::Slab;
 
 pub mod accept;
 pub mod action;
+pub mod close;
 pub mod connect;
 pub mod packet;
 pub mod read;
@@ -20,17 +21,22 @@ pub mod recv;
 pub mod recvmsg;
 pub mod send;
 pub mod sendmsg;
+pub mod shared_fd;
+pub mod socket;
 pub mod stream;
 pub mod timeout;
 pub mod write;
 
 pub use action::Action;
+pub use close::Close;
 pub use packet::Packet;
 pub use read::Read;
 pub use recv::Recv;
 pub use recvmsg::RecvMsg;
 pub use send::Send;
 pub use sendmsg::SendMsg;
+pub use shared_fd::SharedFd;
+pub use socket::Socket;
 pub use stream::Stream;
 pub use timeout::Timeout;
 pub use write::Write;
