@@ -1,22 +1,27 @@
 use std::future::Future;
 use std::io;
-use std::os::unix::io::RawFd;
 use std::pin::Pin;
 use std::task::{Context, Poll};
 
 use io_uring::{opcode, types};
 
-use crate::driver::Action;
+use crate::driver::{Action, SharedFd};
 
 pub struct Read {
+    fd: SharedFd,
     buf: Vec<u8>,
 }
 
 impl Action<Read> {
-    pub fn read(fd: RawFd, len: u32) -> io::Result<Action<Read>> {
-        let mut buf = Vec::with_capacity(len as usize);
-        let entry = opcode::Read::new(types::Fd(fd), buf.as_mut_ptr(), len).build();
-        Action::submit(Read { buf }, entry)
+    pub fn read(fd: &SharedFd, len: u32) -> io::Result<Action<Read>> {
+        let buf = Vec::with_capacity(len as usize);
+        let mut read = Read {
+            fd: fd.clone(),
+            buf,
+        };
+        let entry =
+            opcode::Read::new(types::Fd(read.fd.raw_fd()), read.buf.as_mut_ptr(), len).build();
+        Action::submit(read, entry)
     }
 
     pub fn poll_read(&mut self, cx: &mut Context) -> Poll<io::Result<Vec<u8>>> {
